@@ -18,21 +18,16 @@
 # You should have received a copy of the GNU General Public License
 # along with Rcpp.  If not, see <http://www.gnu.org/licenses/>.
 
-.setUp <- function() {
-
-    tests <- ".Rcpp.Date"
-    if( ! exists(tests, globalenv() )) {
-
-        ## definition of all the functions at once
-        f <- list("ctor_sexp"=list(
-                  signature(d="Date"),
-                  'Date dt = Date(d);
+definitions <- function() {
+    list(          "ctor_sexp"=list(
+                   signature(d="Date"),
+                   'Date dt = Date(d);
                    return wrap(dt);')
 
                   ,"ctor_mdy"=list(
                    signature(),
                    'Date dt = Date(12,31,2005);
-                    return wrap(dt);')
+                   return wrap(dt);')
 
                   ,"ctor_ymd"=list(
                    signature(),
@@ -44,9 +39,15 @@
                    'Date dt = Date(Rcpp::as<int>(d));
                     return wrap(dt);')
 
-                  ,"operators"=list(
-                   signature(),
-                   'Date d1 = Date(2005,12,31);
+                  ,"ctor_string"=list(
+                   signature(x="date"),
+                   'std::string dtstr = Rcpp::as<std::string>(x);
+                    Date dt(dtstr);
+				    return wrap(dt);')
+
+                   ,"operators"=list(
+                    signature(),
+                    'Date d1 = Date(2005,12,31);
                     Date d2 = d1 + 1;
                     return List::create(Named("diff") = d2 - d1,
                                         Named("bigger") = d2 > d1,
@@ -86,17 +87,26 @@
                     v[1] = Date(12,31,2005) ;
                     return wrap( v );')
 
+                  ,"Date_get_functions"=list(
+                   signature(x="Date"),
+                   'Date d = Date(x);
+		            return List::create(Named("year") = d.getYear(),
+    		                            Named("month") = d.getMonth(),
+    		                            Named("day") = d.getDay(),
+    		                            Named("wday") = d.getWeekday(),
+    		                            Named("yday") = d.getYearday());')
+
                   ,"Datetime_get_functions"=list(
                    signature(x="Datetime"),
                    'Datetime dt = Datetime(x);
 		            return List::create(Named("year") = dt.getYear(),
     		                            Named("month") = dt.getMonth(),
-    		                                Named("day") = dt.getDay(),
-    		                                Named("wday") = dt.getWeekday(),
-    		                                Named("hour") = dt.getHours(),
-    		                                Named("minute") = dt.getMinutes(),
-    		                                Named("second") = dt.getSeconds(),
-    		                                Named("microsec") = dt.getMicroSeconds());')
+    		                            Named("day") = dt.getDay(),
+    		                            Named("wday") = dt.getWeekday(),
+    		                            Named("hour") = dt.getHours(),
+    		                            Named("minute") = dt.getMinutes(),
+    		                            Named("second") = dt.getSeconds(),
+    		                            Named("microsec") = dt.getMicroSeconds());')
 
                   ,"Datetime_operators"=list(
                    signature(),
@@ -115,13 +125,20 @@
                    'Datetime dt = Datetime(981162123.123456);
 				    return wrap(dt);')
 
-                  )
+                  ,"Datetime_from_string"=list(
+                   signature(x="datetime"),
+                   'std::string dtstr = Rcpp::as<std::string>(x);
+                    Datetime dt(dtstr);
+				    return wrap(dt);')
 
-        signatures <- lapply(f, "[[", 1L)
-        bodies <- lapply(f, "[[", 2L)
-        fun <- cxxfunction( signatures, bodies, plugin = "Rcpp")
-        getDynLib( fun ) # just forcing loading the dll now
-        assign( tests, fun, globalenv() )
+                  )
+}
+
+.setUp <- function() {
+    tests <- ".Rcpp.Date"
+    if( ! exists(tests, globalenv() )) {
+        fun <- Rcpp:::compile_unit_tests(definitions() )
+        assign( tests, fun , globalenv() )
     }
 }
 
@@ -153,6 +170,16 @@ test.Date.ctor.int <- function() {
     checkException(fun("foo"), msg = "Date.ctor -> exception" )
 }
 
+test.Date.ctor.string <- function() {
+    fun <- .Rcpp.Date$ctor_string
+    dtstr <- "1991-02-03"
+    dtfun <- fun(dtstr)
+    dtstr <- as.Date(strptime(dtstr, "%Y-%m-%d"))
+    ddstr <- as.Date(dtstr, "%Y-%m-%d")
+    checkEquals(dtfun, dtstr, msg = "Date.fromString.strptime")
+    checkEquals(dtfun, ddstr, msg = "Date.fromString.asDate")
+}
+
 test.Date.operators <- function() {
     fun <- .Rcpp.Date$operators
     checkEquals(fun(),
@@ -182,6 +209,16 @@ test.DateVector.operator.SEXP <- function(){
     checkEquals(fun(), rep(as.Date("2005-12-31"),2), msg = "DateVector.SEXP")
 }
 
+test.Date.getFunctions <- function(){
+    fun <- .Rcpp.Date$Date_get_functions
+    checkEquals(fun(as.Date("2010-12-04")),
+                list(year=2010, month=12, day=4, wday=7, yday=338), msg = "Date.get.functions.1")
+    checkEquals(fun(as.Date("2010-01-01")),
+                list(year=2010, month=1, day=1, wday=6, yday=1), msg = "Date.get.functions.2")
+    checkEquals(fun(as.Date("2009-12-31")),
+                list(year=2009, month=12, day=31, wday=5, yday=365), msg = "Date.get.functions.3")
+}
+
 test.Datetime.get.functions <- function() {
     fun <- .Rcpp.Date$Datetime_get_functions
     checkEquals(fun(as.numeric(as.POSIXct("2001-02-03 01:02:03.123456", tz="UTC"))),
@@ -201,3 +238,11 @@ test.Datetime.wrap <- function() {
     checkEquals(as.numeric(fun()), as.numeric(as.POSIXct("2001-02-03 01:02:03.123456", tz="UTC")),
                 msg = "Datetime.wrap")
 }
+
+test.Datetime.fromString <- function() {
+    fun <- .Rcpp.Date$Datetime_from_string
+    dtstr <- "1991-02-03 04:05:06.789"
+    dtfun <- fun(dtstr)
+    dtstr <- as.POSIXct(strptime(dtstr, "%Y-%m-%d %H:%M:%OS"))
+    checkEquals(as.numeric(dtfun), as.numeric(dtstr), msg = "Datetime.fromString")
+}
