@@ -2,7 +2,7 @@
 //
 // Dimension.h: Rcpp R/C++ interface class library -- dimensions
 //
-// Copyright (C) 2010	Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2010 - 2011 Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of Rcpp.
 //
@@ -65,7 +65,12 @@ namespace Rcpp{
 		return std::accumulate( dims.begin(), dims.end(), 1, std::multiplies<int>() ) ;
 	}
 	
-	int& Dimension::operator[](int i) throw(std::range_error){
+	Dimension::reference Dimension::operator[](int i) throw(std::range_error){
+		if( i < 0 || i>=static_cast<int>(dims.size()) ) throw std::range_error("index out of bounds") ;
+		return dims.at(i) ;
+	}
+
+	Dimension::const_reference Dimension::operator[](int i) const throw(std::range_error){
 		if( i < 0 || i>=static_cast<int>(dims.size()) ) throw std::range_error("index out of bounds") ;
 		return dims.at(i) ;
 	}
